"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var js_restful_1 = require("js-restful");
var express = require("express");
var pathUtil = require("./path-util");
var renderers_1 = require("./renderers");
var winston = require("winston");
var namings = require("./namings");
var descriptions_1 = require("./descriptions");
var JsRestfulRegistry = (function () {
    function JsRestfulRegistry(app, config) {
        if (config === void 0) { config = {}; }
        this.app = app;
        this.config = config;
        this.registeredServices = [];
        this.securityContextFactory = null;
        if (!config.logger) {
            config.logger = winston;
        }
    }
    JsRestfulRegistry.prototype.registerSecurityContextFactory = function (factory) {
        this.securityContextFactory = factory;
    };
    JsRestfulRegistry.prototype.registerService = function (service) {
        var _this = this;
        if (typeof service === 'function') {
            throw new TypeError('A type is not allowed - only an object can be registered');
        }
        // check if the service is not already registered at this app
        if (this.registeredServices.indexOf(service.constructor.name) !== -1) {
            throw new Error("A service can only be registered once per app. " + service.constructor.name + " is already resgitered.");
        }
        var descriptions = js_restful_1.ServiceParser.parse(service);
        if (descriptions.isSecurityContextUsed()) {
            if (!this.securityContextFactory) {
                throw new Error('if security features are used you need to register a SecurityContextFactory - ExpressServiceRegistry.registerSecurityContextFactory');
            }
        }
        this.config.logger.info(service.constructor.name + " will be registered");
        // store the service at the app
        this.registeredServices.push(service.constructor.name);
        var router = express.Router();
        descriptions.methods.forEach(function (method) {
            // create a http method name from the enum. the enum are capitalized http method
            // - so, convert to string and convert to lowercase is enough.
            var httpMethodName = js_restful_1.HttpMethod[method.httpMethod].toLowerCase();
            var path = method.path ? method.path : '/';
            _this.config.logger.info("register method " + method.methodName + " for path " + path);
            router[httpMethodName](path, function (req, res, next) {
                try {
                    if (descriptions.isSecurityContextUsed()) {
                        var ctx_1 = _this.securityContextFactory.createSecurityContext(req);
                        if (method.isSecurityContextUsed()) {
                            if (!method.permitAll) {
                                var allowed = method.rolesAllowed.length === 0 || method.rolesAllowed.some(function (role) {
                                    return ctx_1.isUserInRole(role);
                                });
                                if (!allowed) {
                                    res.status(403).send('permission denied');
                                    return;
                                }
                            }
                        }
                        else if (descriptions.permitAll || descriptions.rolesAllowed.length > 0) {
                            if (!descriptions.permitAll) {
                                var allowed = descriptions.rolesAllowed.length === 0 || descriptions.rolesAllowed.some(function (role) {
                                    return ctx_1.isUserInRole(role);
                                });
                                if (!allowed) {
                                    res.status(403).send('permission denied');
                                    return;
                                }
                            }
                        } // else no sec check required
                    }
                    var args = _this.collectAndConvertArgs(req, res, next, service, method);
                    var methodToCall = service[method.methodName];
                    var resultRenderer_1 = renderers_1.RendererFactory.getRenderer(service, method);
                    var result = methodToCall.apply(service, args);
                    if (result && 'function' === typeof result.then) {
                        result.then(function (pResult) {
                            resultRenderer_1.render(res, pResult);
                        });
                        result.catch(function (err) {
                            next(err);
                        });
                    }
                    else {
                        resultRenderer_1.render(res, result);
                    }
                }
                catch (err) {
                    next(err);
                }
            });
        });
        var basePath = pathUtil.getPathFromString(descriptions.basePath);
        this.app.use(basePath, router);
        this.config.logger.info(service.constructor.name + " published at " + basePath);
    };
    JsRestfulRegistry.prototype.collectAndConvertArgs = function (req, res, next, service, method) {
        var _this = this;
        var args = [];
        args.length = method.pathParams.length + method.headerParams.length + method.queryParams.length + method.contextParams.length;
        method.pathParams.forEach(function (pathParam) {
            // this is always a string
            var rawParam = req.params[pathParam.paramName];
            args[pathParam.index] = _this.convertRawParamToMethodParam(service, method, pathParam, rawParam);
        });
        method.headerParams.forEach(function (headerParam) {
            var rawParam = req.header(headerParam.paramName);
            args[headerParam.index] = _this.convertRawParamToMethodParam(service, method, headerParam, rawParam);
        });
        method.queryParams.forEach(function (queryParam) {
            var rawParam = req.query[queryParam.paramName];
            args[queryParam.index] = _this.convertRawParamToMethodParam(service, method, queryParam, rawParam);
        });
        method.contextParams.forEach(function (contextParam) {
            var contextType = js_restful_1.ContextTypes[contextParam.paramName];
            switch (contextType) {
                case js_restful_1.ContextTypes.HttpRequest: {
                    args[contextParam.index] = req;
                    break;
                }
                case js_restful_1.ContextTypes.HttpResponse: {
                    args[contextParam.index] = res;
                    break;
                }
                default: {
                    throw new Error("unsupported contexttype " + contextParam.paramName);
                }
            }
        });
        if (method.securityContextParam) {
            args[method.securityContextParam.index] = this.securityContextFactory.createSecurityContext(req);
        }
        var expressContextParams = Reflect.getMetadata(namings.expressContextParam, service, method.methodName) || [];
        expressContextParams.forEach(function (expressContextParam) {
            var expressContextType = descriptions_1.ExpressContextType[expressContextParam.paramName];
            switch (expressContextType) {
                case descriptions_1.ExpressContextType.HttpNextFunction: {
                    args[expressContextParam.index] = next;
                    break;
                }
                default: {
                    throw new Error("unsupported contexttype " + expressContextParam.paramName);
                }
            }
        });
        return args;
    };
    JsRestfulRegistry.prototype.convertRawParamToMethodParam = function (service, method, pathParam, rawParam) {
        // try to figure out what the method signature expects
        var paramTypes = Reflect.getMetadata('design:paramtypes', service, method.methodName);
        if (paramTypes && paramTypes.length >= pathParam.index) {
            // this is a constructor function of the expected type
            var paramType = paramTypes[pathParam.index];
            return (rawParam === null || rawParam === undefined) ? null : paramType(rawParam);
            ;
        }
        else {
            // ther is no way to figure out what the expected type is - pass it as string
            return rawParam;
        }
    };
    return JsRestfulRegistry;
}());
exports.JsRestfulRegistry = JsRestfulRegistry;
//# sourceMappingURL=registry.js.map